@@ -45,7 +45,6 @@ from .group import Polylogue, Chatroom, ChatroomDataSource
 
 from .ans import AddressNameService
 from .delegate import CipherKeyDelegate
-from .transmitter import Transmitter
 
 from .facebook import Facebook
 from .packer import MessagePacker
@@ -87,6 +86,6 @@ __all__ = [
     # group
     'Polylogue', 'Chatroom', 'ChatroomDataSource',
 
-    'AddressNameService', 'CipherKeyDelegate', 'Transmitter',
+    'AddressNameService', 'CipherKeyDelegate',
     'Facebook', 'Messenger', 'MessagePacker', 'MessageProcessor',
 ]
